ZrMM = {}
ZrMM.Defaults = {
	["MapSize"] = 200,
	["position"] = {
		["point"] = TOPLEFT,
		["relativePoint"] = TOPLEFT,
		["offsetX"] = 0,
		["offsetY"] = 0
	},
	["MapAlpha"] = 100,
	["MapZoom"]  = ZrMM.DEFAULT_ZOOM_LEVEL,
	["MapZoomTown"]  = 12,
	["PinScale"] = 40,
	["hideZoomLabel"] = false,
	["zoneNameColor"] = { ["r"]=1, ["g"]=1, ["b"]=1, ["a"]=1 },
	["zoomLabelColor"] = { ["r"]=1, ["g"]=1, ["b"]=1, ["a"]=1 },
	["hideZoneName"] = false,
	["ZoneNameDock"] = "TOPCENTER",
	["ZoneNameMapDock"] = "BOTTOMCENTER",
	["ZoomLabelDock"] = "TOPLEFT",
	["ZoomLabelMapDock"] = "TOPLEFT",
	["heading"] = "PLAYER",
	["clampToScreen"] = true
}

ZrMM.ZOOM_INCREMENT_AMOUNT = 1
ZrMM.ZOOM_MAX = 20
ZrMM.ZOOM_MIN = 1
ZrMM.DEFAULT_ZOOM_LEVEL = 10

ZrMM.zoPins = {}
ZrMM.pins = {}
ZrMM.noMap = false
ZrMM.wantsShown = true

ZrMM.excludeList = {}
ZrMM.customPins = {}


-----------------------------------------------------------------
-- Updates
-----------------------------------------------------------------
-- delay buffer for updates (thanks Wykkyd :D)
local bt = {}
function MM_DelayBuffer(key, buffer)
	if key == nil then return end
	if bt[key] == nil then bt[key] = {} end
	bt[key].buffer = buffer or 3
	bt[key].now = GetFrameTimeMilliseconds()
	if bt[key].last == nil then bt[key].last = bt[key].now end
	bt[key].diff = bt[key].now - bt[key].last
	bt[key].eval = bt[key].diff >= bt[key].buffer
	if bt[key].eval then bt[key].last = bt[key].now end
	return bt[key].eval
end

function MM_OnUpdate()
	MM_HideCheck()
	if Zr_MM_Scroll:IsHidden() == false then
		MM_MapUpdate()
	end
end

function MM_WorldMapHidden()
	MM_Log("MM_WorldMapHidden()")
	MM_WorldMapUpdate()
end

function MM_ZrMMShown()
	MM_Log("MM_ZrMMShown()")
	MM_MapUpdate()
end

function MM_WorldMapUpdate()
	MM_Log("MM_WorldMapUpdate()")
	SetMapToPlayerLocation()
	ZO_WorldMap_UpdateMap()
end

function MM_HideCheck()
	local menuHidden = ZO_KeybindStripControl:IsHidden()
	local interactHidden = ZO_InteractWindow:IsHidden()
	local gameMenuHidden = ZO_GameMenu_InGame:IsHidden()
	if ZrMM.wantsShown == false or ZrMM.noMap == true then
		MM_Hide()
	elseif menuHidden == true and interactHidden == true and gameMenuHidden == true then
		MM_Show()
	elseif gameMenuHidden == false then
			MM_Show()
	else
		MM_Hide()
	end
end

function MM_Hide()
	Zr_MM_Scroll:SetHidden(true)
	Zr_MM_Bg:SetHidden(true)
	Zr_MM_Zone:SetHidden(true)
	Zr_MM_ZoomLevel:SetHidden(true)
	Zr_MM_Player:SetHidden(true)
	Zr_MM:SetMouseEnabled(false)
end

function MM_Show()
	Zr_MM_Scroll:SetHidden(false)
	Zr_MM_Bg:SetHidden(false)
	Zr_MM_Zone:SetHidden(ZrMM.SV.hideZoneName)
	Zr_MM_ZoomLevel:SetHidden(ZrMM.SV.hideZoomLabel)
	Zr_MM_Player:SetHidden(false)
	Zr_MM:SetMouseEnabled(true)
end

function MM_MapUpdate()
	--MM_Log("MM_MapUpdate()")
	SetMapToPlayerLocation()
	local currentMapName = GetMapName()
	local currentFloor = GetMapFloorInfo()
	if( ZrMM.currentMapName ~= currentMapName ) or ( ZrMM.currentFloor ~= currentFloor ) then
	MM_Log("MM_MapUpdate() - detected mapname change")
		MM_ResetMap()
		MM_PositionUpdate()	
		MM_ZoneUpdate()
		MM_PinCopy()
	else
		MM_PositionUpdate()	
		MM_ZoneUpdate()
		MM_PinUpdate()
	end
end

function MM_ResetMap()
	MM_Log("MM_ResetMap()")
	ZO_WorldMap_UpdateMap()
	--zo_callLater(MM_MapInfo,1)
	
	MM_MapInfo()
	--MM_BuildMapControls()
end

function MM_ZoneUpdate()
	--MM_Log("MM_ZoneUpdate()")
	if Zr_MM_Zone:GetText() ~= GetMapName() then
		
		subzone = GetMapName()
		pos = string.find (subzone, "^", 1, true)
		if pos then
			subzone = string.sub (subzone, 1, pos-1)
		end

		Zr_MM_Zone:SetText(subzone)
		Zr_MM_Zone:SetColor(ZrMM.SV.zoneNameColor.r, ZrMM.SV.zoneNameColor.g, ZrMM.SV.zoneNameColor.b, ZrMM.SV.zoneNameColor.a)
		MM_UpdateZoomLabel()
	end
end

function MM_UpdateZoomLabel()
	MM_Log("MM_UpdateZoomLabel()")
	if ZrMM.SV.hideZoomLabel == true then
		return
	end
	
	if MM_IsMapDungeonOrTown() then
		Zr_MM_ZoomLevel:SetText(ZrMM.SV.MapZoomTown)
	else
		Zr_MM_ZoomLevel:SetText(ZrMM.SV.MapZoom)
	end
end

function MM_PinUpdate()
	if not MM_DelayBuffer("PinUpdate", 500) then return; end
	--d("MM_PinUpdate()")
	MM_PinCopy()
end

-- hides all current pins (fires when Zone Text changes)
function MM_ZoneChange()	
	MM_Log("MM_ZoneChange()")
	local numChildren = Zr_MM_Scroll_Map_Pins:GetNumChildren()
	for pctrlnum=1, numChildren do
		pctrl = Zr_MM_Scroll_Map_Pins:GetChild(pctrlnum)
		if pctrl.Zone ~= GetMapName() then
			pctrl:ClearAnchors()
			pctrl:SetHidden(true)
		end
	end
end


function MM_PositionUpdate()
	--MM_Log("MM_PositionUpdate()")
	local hidePlayer = false;
	local x, y, heading = GetMapPlayerPosition("player")
	if( x > 1 or x < 0) then
		x = 0.5
		hidePlayer = true
	end
	
	if( y > 1 or y < 0 ) then
		y = 0.5
		hidePlayer = true
	end
	
	local mapWidth = Zr_MM_Scroll_Map_0:GetWidth()
	local mmWidth = Zr_MM_Scroll:GetWidth()
	local widthCenter = mmWidth / 2
	local mapWidth = mapWidth * ZrMM.dX
	local mapHeight = Zr_MM_Scroll_Map_0:GetHeight()
	local mmHeight = Zr_MM_Scroll:GetHeight()
	local heightCenter = mmHeight / 2
	local mapHeight = mapHeight * ZrMM.dX 
	local hscroll = x * mapWidth
	local hpos = hscroll - widthCenter
	local vscroll = y * mapHeight
	local vpos = vscroll - heightCenter
	Zr_MM_Scroll:SetHorizontalScroll(hpos)
	Zr_MM_Scroll:SetVerticalScroll(vpos)
	if ZrMM.SV.heading == "PLAYER" then
		Zr_MM_Player:SetTextureRotation(heading)	
	else
		Zr_MM_Player:SetTextureRotation(GetPlayerCameraHeading())
	end
	
	Zr_MM_Player:SetHidden(hidePlayer)
end


-------------------------------------------------------------
-- Map Building
-------------------------------------------------------------
-- get map info for minimap
function MM_MapInfo()
	MM_Log("MM_MapInfo()")
	local currentMap = {} 
	currentMap.tileTexture = GetMapTileTexture()
	
	-- if we have no texture we have nothing further to do
	if currentMap.tileTexture == "" then
		MM_Hide()
		ZrMM.currentMapName = nil
		ZrMM.noMap = true;
		return 
	else
		ZrMM.currentMapName = GetMapName()
		ZrMM.currentFloor = GetMapFloorInfo()
		ZrMM.noMap = false;
	end
	
	ZrMM.dX, ZrMM.dY = GetMapNumTiles()
	currentMap.numTiles = ZrMM.dX * ZrMM.dY
	currentMap.tiles = {}
	
	-- store tile textures in table
	if ( currentMap.numTiles > 1 ) then
		for num = 0, currentMap.numTiles-1 do
			local value = string.gsub( currentMap.tileTexture, '_(%d+).dds', string.format( '_%d.dds', num ) )
			table.insert( currentMap.tiles, value )
		end
	elseif ( currentMap.numTiles == 1 ) then
		table.insert( currentMap.tiles, currentMap.tileTexture )
	end
	
	local xTiles = ZrMM.dX + 1
	-- create tile controls if they don't exist
	local j = 0;
	for i,v in pairs(currentMap.tiles) do
		local ctrlNum = i - 1
		local tileControl = tostring("Zr_MM_Scroll_Map_" .. ctrlNum)
		tCtrl = GetControl(tileControl)
		if tCtrl == nil then
			tCtrl = WINDOW_MANAGER:CreateControl(tileControl, Zr_MM_Scroll_Map, CT_TEXTURE)
		end
		
		-- also go ahead and do the heavy lifting of setting the texture and anchoring everything
		if i == 1 then 
			Zr_MM_Scroll_Map_0:SetTexture(currentMap.tiles[i]) 
		end
		
		tCtrl:SetDimensions(MM_TileScale())
		
		if i == 1 then
			Zr_MM_Scroll_Map_0:SetTexture(currentMap.tiles[i])
		elseif i < xTiles then
			prevCtrlNum = i - 2
			prevCtrl = tostring("_" .. prevCtrlNum)
			prevTileCtrl = Zr_MM_Scroll_Map:GetNamedChild(prevCtrl)
			tCtrl:SetHidden(false)
			tCtrl:SetTexture(currentMap.tiles[i])
			tCtrl:SetAnchor(TOPLEFT,prevTileCtrl,TOPRIGHT,0,0)
		elseif i > ZrMM.dX then
			anchorCtrlNum = ctrlNum - ZrMM.dX
			anchorNum = tostring("_" .. anchorCtrlNum)
			anchorControl = Zr_MM_Scroll_Map:GetNamedChild(anchorNum)
			tCtrl:SetHidden(false)
			tCtrl:SetTexture(currentMap.tiles[i])
			tCtrl:SetAnchor(TOP,anchorControl,BOTTOM,0,0)
		end
		
		j = i
	end

	-- this scares me a bit, I have no idea why it's hardcoded to 50...
	-- is that the current max size of a map?
	-- it's scary as numTiles is actually set by currentMap.Dx * currentMap.Dy
	-- maybe the 50 should be Zr_MM_Scroll_Map:GetNumChildren
	for i=j, 50 do	
		if i > currentMap.numTiles then
			tileNum = i - 1
			tileName = tostring("_" .. tileNum)
			if Zr_MM_Scroll_Map:GetNamedChild(tileName) ~= nil then
				tileControl = Zr_MM_Scroll_Map:GetNamedChild(tileName)
				tileControl:SetHidden(true)
			end
		end
	end
	
	-- when a texture is set for the first time it is not actually loaded
	-- so are calls to get info about the texture will not give us the proper data
	-- so we'll check if it's loaded and if not we'll call the function to set the
	-- dimensions for the map tiles in 1 frame
	if Zr_MM_Scroll_Map_0:IsTextureLoaded() == false then
		zo_callLater(MM_SetMapTilesDimensions, 1)
	else
		MM_SetMapTilesDimensions()
	end
end

-- gets tile scale for map
function MM_TileScale()
	MM_Log("MM_TileScale()")
	local texW, texH = Zr_MM_Scroll_Map_0:GetTextureFileDimensions()
	local dx, dy = GetMapNumTiles()
	local fullMapWidth = texW * dx

	local mapsize = nil
	if MM_IsMapDungeonOrTown() then
		mapsize = (MM_GetMapZoomTown()/10) * fullMapWidth * 0.6
	else
		mapsize = (MM_GetMapZoom()/10) * fullMapWidth
	end
	
	local tileX = mapsize / dx
	local tileY = mapsize / dy
	return tileX,tileY
end

function MM_LoadScreen()
	MM_Log("MM_LoadScreen()")
	MM_OnInit()
	MM_PinCopy()
end

function MM_OnInit()
	MM_Log("MM_OnInit()")
	SetMapToPlayerLocation()
	MM_ResetMap()
end

-- build the minimap
function MM_BuildMapControls()

end

-- Set all our map tiles dimensions to be the proper for the currently loaded texture
function MM_SetMapTilesDimensions()
	MM_Log("MM_SetMapTilesDimensions()")
	local xTiles = ZrMM.dX + 1
	local tileX, tileY = MM_TileScale()
	local length = ZrMM.dX * ZrMM.dY
	--todo clean this up as it's just a quick fix to fix the anchroing issue that we had when this was done in mapinfo
	for i=1, length do
		if i == 1 then
			ctrlNum = i - 1
			tileNum = tostring("_" .. ctrlNum)
			tileControl = Zr_MM_Scroll_Map:GetNamedChild(tileNum)
			tileControl:SetDimensions(tileX, tileY)
		elseif i < xTiles then
			ctrlNum = i - 1
			tileNum = tostring("_" .. ctrlNum)
			tileControl = Zr_MM_Scroll_Map:GetNamedChild(tileNum)
			tileControl:SetDimensions(tileX, tileY)
			prevCtrlNum = i - 2
			prevCtrl = tostring("_" .. prevCtrlNum)
			prevTileCtrl = Zr_MM_Scroll_Map:GetNamedChild(prevCtrl)
			tileControl:SetAnchor(TOPLEFT,prevTileCtrl,TOPRIGHT,0,0)
		elseif i > ZrMM.dX then
			ctrlNum = i - 1
			tileNum = tostring("_" .. ctrlNum)
			tileControl = Zr_MM_Scroll_Map:GetNamedChild(tileNum)
			if tileControl ~= nil then
				tileControl:SetDimensions(tileX, tileY)
			end
			anchorCtrlNum = ctrlNum - ZrMM.dX
			anchorNum = tostring("_" .. anchorCtrlNum)
			anchorControl = Zr_MM_Scroll_Map:GetNamedChild(anchorNum)
			tileControl:SetAnchor(TOP,anchorControl,BOTTOM,0,0)
		end
	end
	
	-- set map dimensions
	local mWidth = tileX * ZrMM.dX
	local mHeight = tileX * ZrMM.dX
	Zr_MM_Scroll_Map:SetDimensions(mWidth, mHeight)
end

-----------------------------------------------------------------
-- Map Pins
-----------------------------------------------------------------
function MM_PinCopy()
	MM_Log("MM_PinCopy()")
	local numChildren = ZO_WorldMapContainer:GetNumChildren()
	ZrMM.zoPins = {}
	for child=1, numChildren do
		childControl = ZO_WorldMapContainer:GetChild(child)
		if childControl.m_Pin ~= nil and childControl.m_Pin:GetUnitTag() ~= 'player' then
			-- do exclude check
			if ZrMM.excludeList[childControl.m_Pin.m_PinType] == nil then
				table.insert(ZrMM.zoPins, childControl)
			end
		end
	end
	
	MM_PinHidden()
	
	local mapWidth, mapHeight = ZO_WorldMapContainer:GetDimensions()
	local dx, dy = GetMapNumTiles()
	local origTileW = mapWidth / dx
	local origTileH = mapHeight / dy

	-- get the tile dimension from our minimap
	local zrmmW, zrmmH = Zr_MM_Scroll_Map_0:GetTextureFileDimensions()
	local zoomFactor = zrmmW / origTileW
	-- apply our zoom
	if MM_IsMapDungeonOrTown() then
		zoomFactor = zoomFactor *(MM_GetMapZoomTown()/10) *0.6
	else
		zoomFactor = zoomFactor *(MM_GetMapZoom()/10)
	end


	local playerZone = GetUnitZone("player")
	local playerName = GetUnitName("player")
	local pinScale = MM_GetPinScale()
	local mapTextureWidth, mapTextureHeight = Zr_MM_Scroll_Map:GetDimensions()
	
	-- one day I'll finish this so pins will be clamped to edge of map
	-- would be helpful for group members and quests
	-- TODO_PIN_CLAMP
--	local playerX, playerY = GetMapPlayerPosition("player")
--	local mapWidth = Zr_MM_Scroll:GetWidth()
--	local mapHeight = Zr_MM_Scroll:GetHeight()
--	local mapHalfWidthNormalized = (mapWidth/mapTextureWidth)/2;
--	local mapHalfHeightNormalized = (mapHeight/mapTextureHeight)/2;
--	local mapBoundsLeft = playerX - mapHalfWidthNormalized --(((playerX * mapTextureWidth) - (mapWidth/2) ) / playerX )
--	local mapBoundsRight = playerX + mapHalfWidthNormalized --mapBoundsLeft + mapWidth;
--	local mapBoundsTop = playerY - mapHalfHeightNormalized --(((playerY * mapTextureHeight) - (mapHeight/2) ) / playerY )
--	local mapBoundsBottom = playerY + mapHalfHeightNormalized --mapBoundsTop + mapHeight;
	--d("-----start")
	--d(mapBoundsLeft)
	--d(mapBoundsRight)
	--d(mapBoundsTop)
	--d(mapBoundsBottom)
	--d("-----end")
	--		if unitTag == 'player' then
				-- let's put this on an edge...
	--			local tX, tY = GetMapPlayerPosition(unitTag)
	--			local tW, tH = Zr_MM_Scroll_Map:GetDimensions()
	--			unitX = (((tX * tW) - 100 ) / tW )
	--			unitY = tY
	--			v:SetHidden(false)
	--			d(unitX)
	--		end
	
	for i,v in pairs(ZrMM.zoPins) do
		if v.m_Pin.normalizedX ~= nil then
		local pinName = v:GetName() -- get ZO_MapPin Name
			-- TODO_PIN_CLAMP
--			if (v.m_Pin.normalizedX > mapBoundsLeft and v.m_Pin.normalizedX < mapBoundsRight and v.m_Pin.normalizedY > mapBoundsTop and v.m_Pin.normalizedY < mapBoundsBottom) then
			
			
			ZrMM.pins[pinName] = {} -- create table to store pin data
			local zrpin = ZrMM.pins[pinName] -- shortcut alias for table
			zrpin.ctrlName = tostring("Zr_MM_Scroll_Map_Pins" .. pinName)
			
			-- if a control with the name exists get it otherwise create one
			if Zr_MM_Scroll_Map_Pins:GetNamedChild(pinName) == nil then
				MM_Log("Creating new control")
				pin = WINDOW_MANAGER:CreateControl(zrpin.ctrlName, Zr_MM_Scroll_Map_Pins, CT_TEXTURE)
			else 
				pin = Zr_MM_Scroll_Map_Pins:GetNamedChild(pinName)
			end
			
			local unitX = nil
			local unitY = nil
			-- check and update if group member pin
			local unitTag = v.m_Pin:GetUnitTag()
			if string.find(unitTag, "group") then
				-- must be a group member
				if GetUnitName(unitTag) ~= playerName then
					-- hey it's not ourselves either...
					if GetUnitZone(unitTag) == playerZone then 
						unitX, unitY = GetMapPlayerPosition(unitTag)
						-- we also want to force the control to not be hidden otherwise user would have to open the map first
						v:SetHidden(false)
						v.hidePin = false
					end
				end
			end
			
			-- TODO_PIN_CLAMP
	--		if unitTag == 'player' then
				-- let's put this on an edge...
	--			local tX, tY = GetMapPlayerPosition(unitTag)
	--			local tW, tH = Zr_MM_Scroll_Map:GetDimensions()
	--			unitX = (((tX * tW) - 100 ) / tW )
	--			unitY = tY
	--			v:SetHidden(false)
	--			d(unitX)
	--		end
			
			-- calculate pin coords from normalized coords
			zrpin.nX = unitX or v.m_Pin.normalizedX
			zrpin.nY = unitY or v.m_Pin.normalizedY
			zrpin.X = zrpin.nX * mapTextureWidth 
			zrpin.Y = zrpin.nY * mapTextureHeight
			
			local originalBack = GetControl(v, "Background")
			local back = GetControl(pin, "Background")
			pin:SetColor(originalBack:GetColor());
			
			zrpin.control = pin
			zrpin.zocontrol = v
			
			local mPin = v.m_Pin
			
			local pinHScale = pinScale;
			local pinWScale = pinScale;
			
			-- get pin texture (thanks to Shinni)
			if mPin.m_PinType ~= nil then
				if mPin.pinBlobKey ~= nil then
					zrpin.texture = mPin.pinBlobTexture
					-- get the dimension of the Quest Circle and apply the zoom factor
					local zoMapPinWidth, zoMapPinHeight = v:GetDimensions()
					pinWScale = zoMapPinWidth * zoomFactor
					pinHScale = zoMapPinHeight * zoomFactor
				elseif type(mPin.PIN_DATA[mPin.m_PinType].texture) == 'string' then
					zrpin.texture = mPin.PIN_DATA[mPin.m_PinType].texture
				elseif type(mPin.PIN_DATA[mPin.m_PinType].texture) == 'function' then
					zrpin.texture = mPin.PIN_DATA[mPin.m_PinType].texture(mPin)
				end
			end
			
			pin:SetHidden(v.hidePin)
			if pin:GetTextureInfo() ~= zrpin.texture then
				pin:SetTexture(zrpin.texture)
			end
			pin:SetDimensions(pinWScale, pinHScale)
			pin:ClearAnchors()
			pin:SetAnchor(CENTER, Zr_MM_Scroll_Map_Pins, TOPLEFT, zrpin.X, zrpin.Y)
			
			if mPin:IsQuest() == true then
				pin:SetDrawLayer(1)
			else
				pin:SetDrawLayer(2)
			end
			
			pin:SetMouseEnabled(true)
			pin.Zone = GetMapName()
		
		end
	end
end

function MM_PinMouseEnter()
	
end


-- checks to see if a pins are hidden on the world map
function MM_PinHidden()
	MM_Log("MM_PinHidden()")
	-- I really hate to do it this way, but I'm have yet to figure out why the pvp pins do not update
	-- until you actually reopen the map. So for now to get it working we'll jsut brute force and hide them
	-- if we're not currently in the map content and type that they display in
	local hidePvpPins = true
	if GetMapContentType() == MAP_CONTENT_AVA and GetMapType() ~= MAPTYPE_SUBZONE then
		hidePvpPins = false
	end
	if ZO_WorldMap:IsHidden() == true then
		ZO_WorldMap:SetAlpha(0)
		ZO_WorldMap:SetHidden(false)
		for i,v in ipairs(ZrMM.zoPins) do
			if v.m_Pin:IsKeep() == true or v.m_Pin:IsAvAObjective() == true then
				v.hidePin = hidePvpPins
			else
				v.hidePin = v:IsHidden()
			end
		end
		ZO_WorldMap:SetAlpha(1)
		ZO_WorldMap:SetHidden(true)
	else
		for i,v in ipairs(ZrMM.zoPins) do
			v.hidePin = v:IsHidden()
		end
	end
end


------------------------------------------------------------
-- Settings
-------------------------------------------------------------
function MM_GetMapSize()
	MM_Log("MM_GetMapSize()")
	local mwidth = Zr_MM_Scroll:GetWidth()
	return mwidth
end

function MM_SetMapSize(newSize)
	MM_Log("MM_SetMapSize()")
	Zr_MM_Scroll:SetDimensions(newSize,newSize)
	Zr_MM:SetDimensions(newSize,newSize)
	ZrMM.SV.MapSize = newSize
end

function MM_GetMapAlpha()
	MM_Log("MM_GetMapAlpha()")
	return ZrMM.SV.MapAlpha
end

function MM_SetMapAlpha(newAlpha)
	MM_Log("MM_SetMapAlpha()")
	local newAlphaPercent = newAlpha / 100
	Zr_MM:SetAlpha(newAlphaPercent)
	ZrMM.SV.MapAlpha = newAlpha
end

function MM_GetPinScale()
	MM_Log("MM_GetPinScale()")
	local scale = ZrMM.SV.PinScale
	return scale
end

function MM_SetPinScale(newScale)
	MM_Log("MM_SetPinScale()")
	if ZrMM.pins ~= nil then
		for i,v in ipairs(ZrMM.pins) do
			local MMpin = v.control
			MMpin:SetDimensions(newScale, newScale)
		end
	end
	ZrMM.SV.PinScale = newScale
end

function MM_GetMapZoom()
	MM_Log("MM_GetMapZoom()")
	local MM_zoom = ZrMM.SV.MapZoom
	if MM_zoom == nil then
		ZrMM.SV.MapZoom = ZrMM.DEFAULT_ZOOM_LEVEL
		MM_zoom = ZrMM.DEFAULT_ZOOM_LEVEL
	end

	return MM_zoom
end

function MM_SetMapZoom(newZoom)
	MM_Log("MM_SetMapZoom()")
	ZrMM.SV.MapZoom = newZoom
	Zr_MM_ZoomLevel:SetText(newZoom)
end

function MM_GetMapZoomTown()
	MM_Log("MM_GetMapZoomTown()")
	local MM_zoomTown = ZrMM.SV.MapZoomTown
	if MM_zoomTown == nil then
		ZrMM.SV.MapZoomTown = 60
		MM_zoomTown = 60
	end
	return MM_zoomTown
end

function MM_SetMapZoomTown(newZoom)
	MM_Log("MM_SetMapZoomTown()")
	ZrMM.SV.MapZoomTown = newZoom
end

function MM_GetHideCompass()
	MM_Log("MM_GetHideCompass()")
	return ZrMM.SV.hideCompass
end

function MM_SetHideCompass(checkbox)
	MM_Log("MM_SetHideCompass()")
	ZO_Compass:SetHidden(checkbox)
	ZrMM.SV.hideCompass = checkbox
end

function MM_GetHideZoomLabel()
	MM_Log("MM_GetHideZoomLabel()")
	return ZrMM.SV.hideZoomLabel
end

function MM_SetHideZoomLabel(checkbox)
	MM_Log("MM_SetHideZoomLabel()")
	Zr_MM_ZoomLevel:SetHidden(checkbox)
	ZrMM.SV.hideZoomLabel = checkbox
	MM_UpdateZoomLabel()
end

function MM_GetHideZoneName()
	MM_Log("MM_GetHideZoneName()")
	return ZrMM.SV.hideZoneName
end

function MM_SetHideZoneName(checkbox)
	MM_Log("MM_SetHideZoneName()")
	Zr_MM_Zone:SetHidden(checkbox)
	ZrMM.SV.hideZoneName = checkbox
end

function MM_GetZoneNameDock()
	MM_Log("MM_GetZoneNameDock()")
	return ZrMM.SV.ZoneNameDock
end

function MM_SetZoneNameDock(dockPosition)
	MM_Log("MM_SetZoneNameDock()")
	ZrMM.SV.ZoneNameDock = dockPosition
	local valid, point, relativeTo, relativePoint = Zr_MM_Zone:GetAnchor(0) -- assume it's are only anchor?
	MM_SetCtrlDockPoint(Zr_MM_Zone, Zr_MM, dockPosition, nil, relativePoint)
end

function MM_GetZoneNameMapDock()
	MM_Log("MM_GetZoneNameMapDock()")
	return ZrMM.SV.ZoneNameMapDock
end

function MM_SetZoneNameMapDock(dockPosition)
	MM_Log("MM_SetZoneNameMapDock()")
	ZrMM.SV.ZoneNameMapDock = dockPosition
	local valid, point, relativeTo, relativePoint = Zr_MM_Zone:GetAnchor(0) -- assume it's are only anchor?
	MM_SetCtrlDockPoint(Zr_MM_Zone, Zr_MM, dockPosition, point, nil)
end

function MM_GetZoomLabelDock()
	MM_Log("MM_GetZoomLabelDock()")
	return ZrMM.SV.ZoomLabelDock
end

function MM_SetZoomLabelDock(dockPosition)
	MM_Log("MM_SetZoomLabelDock()")
	ZrMM.SV.ZoomLabelDock = dockPosition
	local valid, point, relativeTo, relativePoint = Zr_MM_ZoomLevel:GetAnchor(0) -- assume it's are only anchor?
	MM_SetCtrlDockPoint(Zr_MM_ZoomLevel, Zr_MM, dockPosition, nil, relativePoint)
end

function MM_GetZoomLabelMapDock()
	MM_Log("MM_GetZoomLabelMapDock()")
	return ZrMM.SV.ZoomLabelMapDock
end

function MM_SetZoomLabelMapDock(dockPosition)
	MM_Log("MM_SetZoomLabelMapDock()")
	ZrMM.SV.ZoomLabelMapDock = dockPosition
	local valid, point, relativeTo, relativePoint = Zr_MM_ZoomLevel:GetAnchor(0) -- assume it's are only anchor?
	MM_SetCtrlDockPoint(Zr_MM_ZoomLevel, Zr_MM, dockPosition, point, nil)
end

function MM_GetZoneNameColor()
	MM_Log("MM_GetZoneNameColor()")
	return ZrMM.SV.zoneNameColor.r, ZrMM.SV.zoneNameColor.g, ZrMM.SV.zoneNameColor.b, ZrMM.SV.zoneNameColor.a
end

function MM_SetZoneNameColor(r,g,b,a)
	MM_Log("MM_SetZoneNameColor()")
	ZrMM.SV.zoneNameColor.r = r
	ZrMM.SV.zoneNameColor.g = g
	ZrMM.SV.zoneNameColor.b = b
	ZrMM.SV.zoneNameColor.a = a
	Zr_MM_Zone:SetColor(ZrMM.SV.zoneNameColor.r, ZrMM.SV.zoneNameColor.g, ZrMM.SV.zoneNameColor.b, ZrMM.SV.zoneNameColor.a)
end

function MM_GetZoomLabelColor()
	MM_Log("MM_GetZoomLabelColor()")
	return ZrMM.SV.zoomLabelColor.r, ZrMM.SV.zoomLabelColor.g, ZrMM.SV.zoomLabelColor.b, ZrMM.SV.zoomLabelColor.a
end

function MM_SetZoomLabelColor(r,g,b,a)
	MM_Log("MM_SetZoomLabelColor()")
	ZrMM.SV.zoomLabelColor.r = r
	ZrMM.SV.zoomLabelColor.g = g
	ZrMM.SV.zoomLabelColor.b = b
	ZrMM.SV.zoomLabelColor.a = a
	Zr_MM_ZoomLevel:SetColor(ZrMM.SV.zoomLabelColor.r, ZrMM.SV.zoomLabelColor.g, ZrMM.SV.zoomLabelColor.b, ZrMM.SV.zoomLabelColor.a)
end

function MM_GetHeading()
	MM_Log("MM_GetHeading()")
	return ZrMM.SV.heading
end

function MM_SetHeading(newHeading)
	MM_Log("MM_SetHeading()")
	ZrMM.SV.heading = newHeading
end

function MM_GetClampToScreen()
	MM_Log("MM_GetClampToScreen()")
	return ZrMM.SV.clampToScreen
end

function MM_SetClampToScreen(checkbox)
	MM_Log("MM_SetClampToScreen()")
	Zr_MM:SetClampedToScreen(checkbox)
	ZrMM.SV.clampToScreen = checkbox
end

-------------------------------------------------------------
-- On Initialized
-------------------------------------------------------------

function MM_LoadSavedVars()	
	MM_Log("MM_LoadSavedVars()")
	if ZrMM.SV.position ~= nil then
		local pos = {}
		pos.anchorTo = GetControl(pos.anchorTo)
		Zr_MM:SetAnchor(ZrMM.SV.position.point, pos.anchorTo, ZrMM.SV.position.relativePoint, ZrMM.SV.position.offsetX, ZrMM.SV.position.offsetY)
		Zr_MM_Bg:SetAnchorFill(Zr_MM)
	end
	if ZrMM.SV.MapSize ~= nil then MM_SetMapSize(ZrMM.SV.MapSize) end
	if ZrMM.SV.MapAlpha ~= nil then MM_SetMapAlpha(ZrMM.SV.MapAlpha) end
	if ZrMM.SV.MapZoom ~= nil then MM_SetMapZoom(ZrMM.SV.MapZoom) end
	if ZrMM.SV.MapZoomTown ~= nil then MM_SetMapZoomTown(ZrMM.SV.MapZoomTown) end
	if ZrMM.SV.PinScale ~= nil then MM_SetPinScale(ZrMM.SV.PinScale) end
	if ZrMM.SV.hideCompass ~= nil then MM_SetHideCompass(ZrMM.SV.hideCompass) end
	if ZrMM.SV.hideZoomLabel ~= nil then MM_SetHideZoomLabel(ZrMM.SV.hideZoomLabel) end
	if ZrMM.SV.zoneNameColor ~= nil then MM_SetZoneNameColor(ZrMM.SV.zoneNameColor.r, ZrMM.SV.zoneNameColor.g, ZrMM.SV.zoneNameColor.b, ZrMM.SV.zoneNameColor.a) end
	if ZrMM.SV.zoomLabelColor ~= nil then MM_SetZoomLabelColor(ZrMM.SV.zoomLabelColor.r, ZrMM.SV.zoomLabelColor.g, ZrMM.SV.zoomLabelColor.b, ZrMM.SV.zoomLabelColor.a) end
	if ZrMM.SV.hideZoneName ~= nil then MM_SetHideZoneName(ZrMM.SV.hideZoneName) end
	if ZrMM.SV.ZoneNameDock ~= nil then MM_SetZoneNameDock(ZrMM.SV.ZoneNameDock) end
	if ZrMM.SV.ZoneNameMapDock ~= nil then MM_SetZoneNameMapDock(ZrMM.SV.ZoneNameMapDock) end
	if ZrMM.SV.ZoomLabelDock ~= nil then MM_SetZoomLabelDock(ZrMM.SV.ZoomLabelDock) end
	if ZrMM.SV.ZoomLabelMapDock ~= nil then MM_SetZoomLabelMapDock(ZrMM.SV.ZoomLabelMapDock) end
	if ZrMM.SV.clampToScreen ~= nil then MM_SetClampToScreen(ZrMM.SV.clampToScreen) end
end

function MM_OnLoaded(eventCode, addOnName)
	if addOnName ~= "ZrMM" then return end

	ZrMM.SV = ZO_SavedVars:NewAccountWide("ZrMM_SavedVariables", 6, nil, ZrMM.Defaults)
	--ZrMM.SV = ZO_SavedVars:New( "ZrMM_SavedVariables" , 5 , nil , ZrMM.Defaults , nil )
	if ZrMM.SV ~= nil then
		MM_LoadSavedVars()
	end
	
	
	Zr_MM:SetResizeHandleSize(MOUSE_CURSOR_RESIZE_NS)
	Zr_MM:SetHandler("OnMouseUp", function(self) 
		local size = Zr_MM:GetWidth()
		MM_SetMapSize(size)
		ZrMM.SV.position.offsetX = Zr_MM:GetLeft()
		ZrMM.SV.position.offsetY = Zr_MM:GetTop()
		end)
	Zr_MM_Scroll:SetScrollBounding(0)
	Zr_MM_Player:SetTexture("EsoUI//Art/MapPins//UI-WorldMapPlayerPip.dds")
	Zr_MM_Zone:SetHandler("OnTextChanged", MM_ZoneChange)
	
	
	ZO_WorldMapInfo:SetHandler("OnHide", MM_WorldMapUpdate)
	--Zr_MM_Player:SetHandler("OnShow", MM_WorldMapUpdate)
	
	Zr_MM_Bg:SetAlpha(0)
	
	local var anchorOptions = {"TOPLEFT", "TOPCENTER", "TOPRIGHT", "LEFTCENTER", "RIGHTCENTER", "BOTTOMLEFT", "BOTTOMCENTER", "BOTTOMRIGHT"}
	-- options panel
	local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")
	ZrMMpanel = LAM:CreateControlPanel("MM_OptionsPanel", "ZrMiniMap")
		LAM:AddHeader(ZrMMpanel, "MM_OptionsHeader", "Options")
			LAM:AddButton(ZrMMpanel, "MM_Option_ReloadPins", "Reload Pins", "Forces reload of MiniMap pins", MM_ZoneChange)
            LAM:AddSlider(ZrMMpanel, "MM_Option_Size", "MiniMap Size", "Change the size of the MiniMap", 100, 1000,10, MM_GetMapSize, MM_SetMapSize)
			LAM:AddSlider(ZrMMpanel, "MM_Option_Alpha", "MiniMap Alpha", "Change the transparency of the MiniMap", 0, 100,1, MM_GetMapAlpha, MM_SetMapAlpha)
			LAM:AddSlider(ZrMMpanel, "MM_Option_PinScale", "Pin Scale", "Adjust the scale of MiniMap pins", 0, 100,1, MM_GetPinScale, MM_SetPinScale)
			LAM:AddSlider(ZrMMpanel, "MM_Option_Zoom", "MiniMap Zoom", "Change the zoom of the minimap", ZrMM.ZOOM_MIN, ZrMM.ZOOM_MAX, ZrMM.ZOOM_INCREMENT_AMOUNT, MM_GetMapZoom, MM_SetMapZoom)
			LAM:AddSlider(ZrMMpanel, "MM_Option_Zoom_TownDungeon", "MM Zoom Town and Dungeon", "Change the zoom of the minimap for town and dungeons", ZrMM.ZOOM_MIN, ZrMM.ZOOM_MAX, ZrMM.ZOOM_INCREMENT_AMOUNT, MM_GetMapZoomTown, MM_SetMapZoomTown)
			LAM:AddCheckbox(ZrMMpanel, "MM_Option_HideCompass", "Hide Compass", "Hide the default UI's compass", MM_GetHideCompass, MM_SetHideCompass)
			LAM:AddCheckbox(ZrMMpanel, "MM_Option_HideZoomLabel", "Hide Zoom Label", "Hide the zoom label", MM_GetHideZoomLabel, MM_SetHideZoomLabel)
			LAM:AddDropdown(ZrMMpanel, "MM_Option_ZoomLabelDock", "Zoom Label Anchor Point", "Zoom label anchor point that attaches to \"Zoom Label Map Anchor Point\"", anchorOptions, MM_GetZoomLabelDock, MM_SetZoomLabelDock)
			LAM:AddDropdown(ZrMMpanel, "MM_Option_ZoomLabelMapDock", "Zoom Label Map Anchor Point", "Map anchor point that zoom label attaches to", anchorOptions, MM_GetZoomLabelMapDock, MM_SetZoomLabelMapDock)
			LAM:AddColorPicker(ZrMMpanel, "MM_Option_ZoomLabelColor", "Color of Zoom Label", "Choose what color the zoom level will be displayed", MM_GetZoomLabelColor, MM_SetZoomLabelColor)
			LAM:AddCheckbox(ZrMMpanel, "MM_Option_HideZoneName", "Hide Zone Name", "Hide the zone name", MM_GetHideZoneName, MM_SetHideZoneName)
			LAM:AddDropdown(ZrMMpanel, "MM_Option_ZoneNameDock", "Zone Name Anchor Point", "Zone label anchor point that attaches to \"Zone Name Map Anchor Point\"", anchorOptions, MM_GetZoneNameDock, MM_SetZoneNameDock)
			LAM:AddDropdown(ZrMMpanel, "MM_Option_ZoneNameMapDock", "Zone Name Map Anchor Point", "Map anchor point that zone name attaches to", anchorOptions, MM_GetZoneNameMapDock, MM_SetZoneNameMapDock)
			LAM:AddColorPicker(ZrMMpanel, "MM_Option_ZoneNameColor", "Color of Zone Name", "Choose what color the zone name will be displayed", MM_GetZoneNameColor, MM_SetZoneNameColor)
			LAM:AddDropdown(ZrMMpanel, "MM_Option_Heading", "MiniMap Heading", "Player pin uses characters heading or camera heading", {"CAMERA", "PLAYER"}, MM_GetHeading, MM_SetHeading)
			LAM:AddCheckbox(ZrMMpanel, "MM_Option_ClampToScreen", "Clamp Map To Screen", "Prevents dragging the map offscreen", MM_GetClampToScreen, MM_SetClampToScreen)
			
	Zr_MM:SetHandler("OnUpdate", MM_OnUpdate)
end

-------------------------------------------------------------
-- Bindables
-------------------------------------------------------------

function ZrMM_ZoomOut()
	MM_Log("ZrMM_ZoomOut()")
	local getZoomFunc = nil
	local setZoomFunc = nil
	local zoomLevel = nil
	
	if MM_IsMapDungeonOrTown() then
		zoomLevel = MM_GetMapZoomTown()
		setZoomFunc = MM_SetMapZoomTown
	else
		zoomLevel = MM_GetMapZoom()
		setZoomFunc = MM_SetMapZoom
	end
	
	local zoomLevel = zoomLevel - ZrMM.ZOOM_INCREMENT_AMOUNT
	if zoomLevel < ZrMM.ZOOM_MIN then zoomLevel = ZrMM.ZOOM_MIN end
	
	setZoomFunc(zoomLevel)
	MM_OnInit()
	MM_PinCopy()
end

function ZrMM_ZoomIn()
	MM_Log("ZrMM_ZoomIn()")
	local setZoomFunc = nil
	local zoomLevel = nil

	if MM_IsMapDungeonOrTown() then
		zoomLevel = MM_GetMapZoomTown()
		setZoomFunc = MM_SetMapZoomTown
	else
		zoomLevel = MM_GetMapZoom()
		setZoomFunc = MM_SetMapZoom
	end
	
	local zoomLevel = zoomLevel + ZrMM.ZOOM_INCREMENT_AMOUNT
	if zoomLevel > ZrMM.ZOOM_MAX then zoomLevel = ZrMM.ZOOM_MAX end
	
	setZoomFunc(zoomLevel)
	MM_OnInit()
	MM_PinCopy()
end

function ZrMM_ToggleVisibility()
	ZrMM.wantsShown = not ZrMM.wantsShown
end

-------------------------------------------------------------
-- Helpers
-------------------------------------------------------------

function MM_IsMapDungeonOrTown()
	MM_Log("MM_IsMapDungeonOrTown()")
-- UIMapType
--    MAPTYPE_NONE		= 0
--    MAPTYPE_SUBZONE	= 1	(town in cyrodil and on standard maps)
--    MAPTYPE_ZONE 		= 2	(Cyrodil, standard maps)
--    MAPTYPE_WORLD		= 3
--    MAPTYPE_ALLIANCE	= 4
--    MAPTYPE_COSMIC	= 5

-- MapContentType
--    MAP_CONTENT_NONE 		= 0
--    MAP_CONTENT_AVA		= 1
--    MAP_CONTENT_DUNGEON	= 2

	local mapType = GetMapType()   
	local mapContentType = GetMapContentType()
	if (mapType == MAPTYPE_SUBZONE) or (mapContentType == MAP_CONTENT_DUNGEON) then
		return true
	else
		return false
	end
end

function MM_GetCtrolDockPoint(dockPosition)
	MM_Log("MM_GetCtrolDockPoint()")
	if dockPosition == "TOPLEFT" then
		return TOPLEFT
	elseif dockPosition == "TOPCENTER" then
		return TOP
	elseif dockPosition == "TOPRIGHT" then
		return TOPRIGHT
	elseif dockPosition == "LEFTCENTER" then
		return LEFT
	elseif dockPosition == "RIGHTCENTER" then
		return RIGHT
	elseif dockPosition == "BOTTOMLEFT" then
		return BOTTOMLEFT
	elseif dockPosition == "BOTTOMCENTER" then
		return BOTTOM
	elseif dockPosition == "BOTTOMRIGHT" then
		return BOTTOMRIGHT
	end
end

function MM_SetCtrlDockPoint(ctrl, ctrlTo, dockPosition, point, relativePoint)
	MM_Log("MM_SetCtrlDockPoint()")
	local newPoint = MM_GetCtrolDockPoint(dockPosition)
	if( relativePoint == nil ) then
		relativePoint = newPoint
	else
		point = newPoint
	end
	ctrl:ClearAnchors()
	ctrl:SetAnchor(point, ctrlTo, relativePoint)
end

function MM_Pulse(cntrlName)

	control = Zr_MM_Scroll_Map_Pins:GetChild(cntrlName)
	local highlightControl = GetControl(control, "Highlight")
	local overlayTexture, pulseTexture, glowTexture = GetPinTextureData(self, singlePinData.texture)

	if(overlayTexture ~= "") then
		overlayControl:SetTexture(overlayTexture)
	end

	if(pulseTexture) then
		self:ResetAnimation(RESET_ANIM_ALLOW_PLAY, LONG_LOOP_COUNT, pulseTexture, overlayTexture, true)
	end
end

function MM_Log(message)
	if false then d(message) end
end

-- clamp pin to edge of map exploration
function MM_TI()
	local p1 = {}
	local p2 = {}
	p1.x, p1.y = GetMapPlayerPosition("player") -- player is in middle of map
	
	-- make a point outside of the square to test...
	p2.x = p1.x + 1;
	p2.y = p1.y + 0.05;
	
	-- line for player to point
	local line1 = line(p1, p2)
	
	-- get map points
	local mapTextureWidth, mapTextureHeight = Zr_MM_Scroll_Map:GetDimensions()
	local mapWidth = Zr_MM_Scroll:GetWidth()
	local mapHeight = Zr_MM_Scroll:GetHeight()
	local mapHalfWidthNormalized = (mapWidth/mapTextureWidth)/2;
	local mapHalfHeightNormalized = (mapHeight/mapTextureHeight)/2;
	local mapBoundsLeft = p1.x - mapHalfWidthNormalized
	local mapBoundsRight = p1.x + mapHalfWidthNormalized
	local mapBoundsTop = p1.y - mapHalfHeightNormalized
	local mapBoundsBottom = p1.y + mapHalfHeightNormalized
	
	local ul = { 	x = mapBoundsLeft,
					y = mapBoundsTop }
	local ur = { 	x = mapBoundsRight,
					y = mapBoundsTop }
	local br = { 	x = mapBoundsRight,
					y = mapBoundsBottom }
	local bl = { 	x = mapBoundsLeft,
					y = mapBoundsBottom }
	
	local top = line(ul,ur)
	local right = line(ur,br)
	local bottom = line(bl,br)
	local left = line(ul,bl)
	
	local ix, iy
	
	local minx = zo_min(p1.x, p2.x)
	local maxx = zo_max(p1.x, p2.x)
	local miny = zo_min(p1.y, p2.y)
	local maxy = zo_max(p1.y, p2.y)

	ix, iy = intersect(line1, top)
	if ix ~= nil and minx <= ix and ix <= maxx then
		-- intersect point is on line
		d("intersect point is on top")
	end
	
	ix, iy = intersect(line1, right)
	if iy ~= nil and miny <= iy and iy <= maxy then 
		-- intersect point is on line
		d("intersect point is on right")
	end
	
	ix, iy = intersect(line1, bottom)
	if ix ~= nil and minx <= ix and ix <= maxx then
		-- intersect point is on line
		d("intersect point is on bottom")
	end
	
	ix, iy = intersect(line1, left)
	if iy ~= nil and miny <= iy and iy <= maxy then 
		-- intersect point is on line
		d("intersect point is on left")
	end
	
end

function line(point1, point2)
	local x = point1.x - point2.x
	local y = point2.y - point1.y
	return { 
				x = x,
				y = y,
				c = y * point1.x + x * point1.y 
			}
end

function intersect(line1, line2)
	local det = line1.y * line2.x - line2.y * line1.x
	local x, y
	if det == 0 then 
		x = nil
		y = nil
	else
		x = ( line2.x * line1.c - line1.x * line2.c ) / det
		y = ( line1.y * line2.c - line2.y * line1.c ) / det
	end
	 
	return x, y
end

-- adding Function to handle Group Events
function MM_GroupEvent()
	ZO_WorldMap_RefreshGroupPins()
	MM_WorldMapUpdate()
end


function MM_PlayerActivated()
	EVENT_MANAGER:UnregisterForEvent( "ZrMiniMapOnInitialPlayerActivated", EVENT_PLAYER_ACTIVATED  , MM_PlayerActivated )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnPlayerActivated", EVENT_PLAYER_ACTIVATED  , MM_LoadScreen )
	
	-- quest events
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnQuestAdvanced", EVENT_QUEST_ADVANCED  , MM_WorldMapUpdate )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnQuestConditionCounterChanged", EVENT_QUEST_CONDITION_COUNTER_CHANGED  , MM_WorldMapUpdate )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnQuestRemoved", EVENT_QUEST_REMOVED  , MM_WorldMapUpdate )

	-- group events
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnUnitCreated", EVENT_UNIT_CREATED  , MM_GroupEvent )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnUnitDestroyed", EVENT_UNIT_DESTROYED  , MM_GroupEvent )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnGroupDisbanded", EVENT_GROUP_DISBANDED  , MM_GroupEvent )
	EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnLeaderUpdated", EVENT_LEADER_UPDATE  , MM_GroupEvent )
	
	-- now call loadscreen to initialize the map proper like, then our group event in case we log in with a group and missed the event
	MM_LoadScreen()
	MM_GroupEvent()
end

-- EXTERNAL FUNCTIONS
function ZrMM_RegisterCustomPin(pinType, name)
	ZrMM.customPins[pinType] = name
end

function ZrMM_UnregisterCustomPin(pinType)
	ZrMM.customPins[pinType] = nil
	ZrMM_ExcludePinType(pinType, false)
end

function ZrMM_ExcludePinType(pinType, exclude)
	if exclude == true then
		ZrMM.excludeList[pinType] = true
		-- when adding item we need to clear the current pins
		local numChildren = Zr_MM_Scroll_Map_Pins:GetNumChildren()
		for pctrlnum=1, numChildren do
			pctrl = Zr_MM_Scroll_Map_Pins:GetChild(pctrlnum)
			pctrl:ClearAnchors()
			pctrl:SetHidden(true)
		end
	else
		ZrMM.excludeList[pinType] = nil
		-- we do not have to do zone change as new pins will be visible fine. it's hiding existing ones that is the problem.
	end
end

EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnLoaded", EVENT_ADD_ON_LOADED , MM_OnLoaded )
EVENT_MANAGER:RegisterForEvent( "ZrMiniMapOnInitialPlayerActivated", EVENT_PLAYER_ACTIVATED  , MM_PlayerActivated )

--EVENT_MANAGER:RegisterForEvent("SomeTestUpdate2", EVENT_ZONE_CHANGED, function(id, zoneName, subZoneName, newSubzone) d( "EVENT_ZONE_CHANGED id: " .. id .. ", zoneName: " .. zoneName .. ", subZoneName: " .. subZoneName ) end)

